<?php
// $Id$
global $base_url;
$dita_path = $base_url .'/'. drupal_get_path('module', 'dita');
define('DITA_CONCEPT_DOCTYPE', 'concept');
define('DITA_CONCEPT_EXTERNAL_SUBSET_PUBLIC_IDENTIFIER',
  '-//OASIS//DTD DITA Concept//EN');
define('DITA_CONCEPT_EXTERNAL_SUBSET_SYSTEM_IDENTIFIER',
   $dita_path .'/dtd1.2/technicalContent/dtd/concept.dtd');

define('DITA_TASK_DOCTYPE', 'task');
define('DITA_TASK_EXTERNAL_SUBSET_PUBLIC_IDENTIFIER',
  '-//OASIS//DTD DITA Task//EN');
define('DITA_TASK_EXTERNAL_SUBSET_SYSTEM_IDENTIFIER',
  $dita_path .'/dtd1.2/technicalContent/dtd/task.dtd');

define('DITA_REFERENCE_DOCTYPE', 'reference');
define('DITA_REFERENCE_EXTERNAL_SUBSET_PUBLIC_IDENTIFIER',
  '-//OASIS//DTD DITA Reference//EN');
define('DITA_REFERENCE_EXTERNAL_SUBSET_SYSTEM_IDENTIFIER',
  $dita_path .'/dtd1.2/technicalContent/dtd/reference.dtd');
//@Todo test reference while there were no samples for it in OT

define('DITA_TO_EMEDDED_XHTML_DIR', 'libs/xslt');
define('DITA_TO_EMEDDED_XHTML_FILENAME', 'dita2xhtml_emb.xsl');

/**
 * Transforms a dita topic to xhtml
 *
 * @param string $xml valid xml string
 * @return string valid xml
 */
function dita_filter_transform($xml) {
  if (!$xml) {
    return FALSE;
  }
  module_load_include('export.inc', 'dita');

  $xslt_dir = drupal_get_path('module', 'dita') . '/' . DITA_TO_EMEDDED_XHTML_DIR;
  if (file_exists($xslt_dir . '/'. DITA_TO_EMEDDED_XHTML_FILENAME)) {
    $xml = dita_apply_xslt($xml, file_get_contents($xslt_dir .
            '/'. DITA_TO_EMEDDED_XHTML_FILENAME), TRUE, $xslt_dir);
  }
  else {
    drupal_set_message('XSLT file not found!', 'error');
  }
  return $xml;
}


/**
 * Sets the subset identifiers for a dita topic, to avoid problems with relative
 * paths to the dtds
 *
 * Use only for XSLT transformations (XSLTProcessor requires valid DTD in the
 *   target xml)
 *
 * @param string $xml valid xml string
 * @param string if not FALSE, forces the doctype to be 'concept'/'reference'/'task'
 * @return string valid xml string
 */
function _dita_rewrite_dtd($xml, $forced_doctype = FALSE) {
  module_load_include('export.inc', 'dita');

  if (!$forced_doctype) {
    $doctype = dita_get_doctype($xml);
  }
  else {
    $doctype = $forced_doctype;
  }

  switch ($doctype) {
    case 'concept':
      $public_id = DITA_CONCEPT_EXTERNAL_SUBSET_PUBLIC_IDENTIFIER;
      $system_id = DITA_CONCEPT_EXTERNAL_SUBSET_SYSTEM_IDENTIFIER;
    break;
    case 'task':
      $public_id = DITA_TASK_EXTERNAL_SUBSET_PUBLIC_IDENTIFIER;
      $system_id = DITA_TASK_EXTERNAL_SUBSET_SYSTEM_IDENTIFIER;
    break;
    case 'reference':
      $public_id = DITA_REFERENCE_EXTERNAL_SUBSET_PUBLIC_IDENTIFIER;
      $system_id = DITA_REFERENCE_EXTERNAL_SUBSET_SYSTEM_IDENTIFIER;
    break;
    default:
      //if we can't recognize the doctype of the document, the most safe thing
      //to do is to not do anything
      return FALSE;
  }
  $xml_object = @dita_create_xml_object($xml, $doctype, $public_id, $system_id);
  if (!$xml_object) {
    return FALSE;
  }
  return $xml_object->saveXML();
}

/**
 * Helper function to validate a given XML with an XSD.
 *
 * @param string $xml
 * @param string $schema_path
 * @return array Empty array is valid.
 */
function _dita_validate_xsd($xml, $schema_path) {
  libxml_clear_errors();

  $dom = new DOMDocument();
  $dom->loadXML($xml);
  //@todo set path?
  $dom->schemaValidate($schema_path);

  return libxml_get_errors();
}

/**
 * Function checks if a string is a valid DITA Topic.
 *
 * @param string $xml XML Document supposed to be a valid DITA Topic
 * @return array
 *   return value of libxml_get_errors()
 */
function dita_is_invalid($xml) {
  return _dita_validate_xsd($xml, dita_get_schema_path());
}

/**
 * Function checks if a string is a valid DITA map.
 *
 * @param string $xml XML Document supposed to be a valid DITA Map
 * @return array
 *   return value of libxml_get_errors()
 */
function dita_map_is_invalid($xml) {
  return _dita_validate_xsd($xml, dita_get_mapschema_path());
}
